# coding: utf-8

import time
import datetime
import logging
import pymongo
import machines

DB_NAME = 'junta'
db = pymongo.Connection()[DB_NAME]

def get_recipes_on_server(server_id):
    """ Возвращает список id всех рецептов, когда-либо устанавливаемых на сервер
    """
    recipes = set()
    for r in db.installed.find({'server_id': server_id}):
        recipes.add(r['recipe_id'])
    return recipes

def create(installation_id, server_id, recipe_id, recipe_version, config):
    """ Записывает в бд запрос на новую инсталляцию.
    """
    db.installed.save({'_id': installation_id,
                       'server_id': server_id,
                       'recipe_id': recipe_id,
                       'recipe_version': recipe_version,
                       'date': time.time(),
                       'status': 'sent',
                       'config': config})

def set_status(server_id, status):
    """ Устанавливает новый статус инсталляции.
        Для какой инсталляции, смотрит recent_installation у агента
    """
    installation_id = machines.get_recent_installation(server_id)
    if installation_id == 0:
        logging.error("error in set_installation_status")
        return
    to_change = db.installed.find_one({'_id': installation_id})
    to_change['status'] = status
    db.installed.save(to_change)

def get_by_server_and_recipe(server_id, recipe_id):
    """ Возвращает инсталляции определенного рецепта на определенном сервере
        (список словарей)
    """
    installations = db.installed.find({'server_id': server_id, 'recipe_id': recipe_id}).sort('date', direction=pymongo.DESCENDING)
    res = [dict(i) for i in installations]
    for r in res:
        r['id'] = str(r['_id'])
        r['date'] = str(datetime.datetime.fromtimestamp(r['date']))[:19]
        r['log'] = get_log(r['id'])
    return res
    
def save_log(server_id, msg_type, msg, n):
    """ Сохраняет сообщение в лог.
    """
    installation_id = machines.get_recent_installation(server_id)
    if installation_id == 0:
        logging.error("error in save_log")
        return
    db.log.save({'installation_id': installation_id,
                 'type': msg_type,
                 'message': msg,
                 'n': n})
    
def get_log(installation_id):
    """  Получение лога по его id
    """
    def cmp_func(a, b):
        if int(a[0]) > int(b[0]):
            return 1
        elif int(b[0]) == int(a[0]):
            return 0
        else:
            return -1
    
    res = db.log.find({'installation_id': installation_id})
    log = []
    for r in res:
        log.append((r['n'], r['type'], r['message']))
    log.sort(cmp=cmp_func)
    return log

def get_sorted_by_time(server_id):
    """ Возвращает инсталляций на сервере, сортированный по времени
    """
    installations = db.installed.find({'server_id': server_id}).sort('date', direction=pymongo.DESCENDING)
    return [dict(i) for i in installations]