# coding: utf-8

try:
    import cgitb
    import datetime
    import logging
    import sys
    import uuid
    
    import machines
    import recipes
    
    import db
except Exception as e:
    logging.error(e)
    logging.error(cgitb.text(sys.exc_info()))


def get_installations(server_id):
    """ Возвращает все инсталляции на сервере
        список кортежей (id рецепта, имя рецепта, список инстлляций)
    """
    recipes_on_server = db.get_recipes_on_server(server_id)
    installations = []
    for recipe_id in recipes_on_server:
        installations.append((recipe_id, recipes.get_name(recipe_id), db.get_by_server_and_recipe(server_id, recipe_id)))
    return installations

def get_recent_installation_log(server_id):
    """  Получение лога последней установки на сервер
    """
    installation_id = machines.get_recent_installation(server_id)
    if installation_id == 0:
        logging.error("error in get_recent_installation_log")
        return
    return db.get_log(installation_id)

def get_by_time(server_id):
    """ Возвращает инсталляции определенного рецепта на определенном сервере
        (список словарей)
    """
    res = db.get_sorted_by_time(server_id)
    for r in res:
        r['id'] = str(r['_id'])
        r['date'] = str(datetime.datetime.fromtimestamp(r['date']))[:19]
        r['log'] = db.get_log(r['id'])
        r['recipe_name'] = recipes.get_name(r['recipe_id'])
    return res

def create(server_id, recipe_id, recipe_version, config, hidden_config_keys):
    """ Записывает в бд запрос на новую инсталляцию.
        Принимает: 
            server_id - id агента
            recipe_id - id рецепта
            recipe_version -  номер версии рецета
            config - словарь из параметров установки
            hidden_config_keys -список имен ключей, которые не надо записывать в бд
    """
    db_config = {}
    db_config.update(config)
    for key in hidden_config_keys:
        del db_config[key]
    installation_id = str(uuid.uuid4()).replace('-', '_')
    db.create(installation_id, server_id, recipe_id, recipe_version, db_config)
    machines.set_recent_installation(server_id, installation_id)