# coding: utf-8

import installations
import recipes
import db
import servertoagent
from sharedvariables import agents_list

""" Используется во views """
def get_list():
    """ Возвращает список словарей из id, name всех агентов
    """
    return [{'id': a,
            'name': db.get_name(a),
            'status': get_status(a),
            'log': installations.get_recent_installation_log(a)} for a in db.get_ids()]

def get_status(machine_id):
    if machine_id in agents_list:
        return db.get_status(machine_id)
    else:
        return 'Offline'

def get_name(machine_id):
    return db.get_name(machine_id)

def get_level(machine_id):
    return db.get_level(machine_id)

def set_level(machine_id, level):
    db.set_level(machine_id, level)

def rename(machine_id, new_name):
    db.rename(machine_id, new_name)
    
def delete(machine_id):
    db.delete(machine_id)
    
def get_all_groups():
    return db.get_all_groups()

def get_groups(machine_id):
    return db.get_groups(machine_id)

def get_groups_not_in(machine_id):
    return db.get_groups_not_in(machine_id)

def set_group(machine_id, group_name):
    db.set_group(machine_id, group_name)
    
def remove_from_group(machine_id, group_name):
    db.remove_from_group(machine_id, group_name)
    
def get_machines_in_group(group_name):
    return db.get_machines_in_group(group_name)

def get_attrs(machine_id):
    return db.get_attrs(machine_id)

def get_attrs_as_dict(machine_id):
    res = {}
    for name, default in db.get_attrs(machine_id):
        res[name] = default
    return res

def set_attr(machine_id, attr_name, default=''):
    db.set_attr(machine_id, attr_name, default)

def delete_attr(machine_id, attr_name):
    db.delete_attr(machine_id, attr_name)
    
""" Используется в installations """
def get_recent_installation(machine_id):
    return db.get_recent_installation(machine_id)
    
def set_recent_installation(machine_id, installation_id=0):
    db.set_recent_installation(machine_id, installation_id)
    
def get_agent(machine_id):
    if machine_id in agents_list:
        return agents_list[machine_id]
    else:
        return None
    
def send_installation_request(machine_id, recipe_id, recipe_version, config, hidden_config_keys):
    agent = get_agent(machine_id)
    if agent is None:
        return False
    installations.create(machine_id, recipe_id, recipe_version, config, hidden_config_keys)
    agent.reset_counter()
    agent.write_message(servertoagent.installation_request(recipes.get_recipe_content(recipe_id, recipe_version),
                                                           recipes.get_setup_tools(),
                                                           config))
    return True

def send_exit_request(machine_id):
    agent = get_agent(machine_id)
    if agent is None:
        return False
    agent.write_message(servertoagent.exit_request())
    return True