# coding: utf-8

import pymongo
import uuid

DB_NAME = 'junta'
db = pymongo.Connection()[DB_NAME]

def exists(recipe_name):
    """ Есть ли уже рецепт с таким именем?
    """
    recipe = db.recipes.find_one({'name': recipe_name})
    return recipe is not None  

def create(recipe_name):
    """ Сохраняет новый рецепт в базу данных, генерирует id.
        Если рецепт с таким именем существует, возвращает None
    """
    recipe = db.recipes.find_one({'name': recipe_name})
    if not recipe:
        recipe_id = str(uuid.uuid4()).replace('-', '_')
        db.recipes.save({'_id': recipe_id, 'name': recipe_name})
        return recipe_id
    else:
        return None    

def get_name(recipe_id):
    """ Возвращает имя рецепта по его id.
        Если рецепт с таким id не существует, возвращает None
    """
    recipe = db.recipes.find_one({'_id': recipe_id})
    if recipe: 
        return recipe['name']
    else:
        return "Удален"
    
def get_id(recipe_name):
    """ Возвращает id рецепта по его имени.
        Если рецепт с таким именем не существует, возвращает None
    """
    recipe = db.recipes.find_one({'name': recipe_name})
    if recipe: 
        return recipe['_id']
    else:
        return None

def get_names():
    """ Возвращает список из имен всех рецептов
    """
    recipes = db.recipes.find()
    return [a['name'] for a in recipes]

def rename(recipe_id, new_name):
    """ Переименнование рецепта с заданным id.
        Если рецепт с таким именем существует, возвращает None и не переименновывает
    """
    check_recipe = db.recipes.find_one({'name': new_name})
    if check_recipe:
        return None  
    recipe = db.recipes.find_one({'_id': recipe_id})
    recipe['name'] = new_name
    db.recipes.save(recipe) 

def delete(recipe_id):
    """ Удаление рецепта по его id
    """
    db.recipes.remove({'_id': recipe_id})
    
def get_recipes_list():
    """ Возвращает список кортежей (id, name) всех рецептов
    """
    return [(x['_id'], x['name']) for x in db.recipes.find()]