# -*- coding: utf-8 -*-

try:
    import cgitb
    import sys
    import logging
    
    from django.core.context_processors import csrf
    from django.contrib.auth.views import login, logout
    from django.contrib import auth
    from django.http import HttpResponseRedirect
    from django.shortcuts import render_to_response
    from django.contrib.auth.decorators import login_required
    
    from users.views import get_user_level                    
    import junta_xml
    from machines.settings import PROTOCOL
    
    import machines
    import recipes
    import installations
except Exception as e:
    logging.error(e)
    logging.error(cgitb.text(sys.exc_info())) 

def setup_logger(log_file, rotation=5*1024, count=5):
    """ Настраивает логирование в STDOUT и указанный лог-файл @a log_file.
        Объём ротации задаётся 
    """
    formatter = logging.Formatter('[%(asctime)s][%(levelname)s][%(module)s]: %(message)s', datefmt='%H:%M:%S %d-%m-%Y')
    
    handler = logging.handlers.RotatingFileHandler(log_file, maxBytes=rotation*1024, backupCount=count)
    handler.setFormatter(formatter)
    logging.getLogger().addHandler(handler)
    
    console = logging.StreamHandler()
    console.setFormatter(formatter)
    logging.getLogger().addHandler(console)
    logging.getLogger().setLevel(logging.DEBUG)
    logging.getLogger('django.db.backends').setLevel(logging.ERROR) 

def server_run():          
    """ Запуск сервера, который связывается с агентами, если он не запущен. 
    """ 
    if machines.server_state != 'started': 
        try:
            setup_logger('/var/log/junta-webserver.log')
        except IOError:
            print >>sys.stderr, 'Could not setup logging to file'
        machines.start_server()
        machines.server_state = 'started'
        logging.info('Server started.')

server_run()
        
def get_common_context(request):
    """ Получение общего контента для всех страниц.
    """ 
    try:
        c = {}
        c.update(csrf(request))
        c['user'] = request.user
        c['recipes'] = recipes.get_recipes()
        c['servers'] = machines.get_list()
        c['protocol'] = PROTOCOL
        c['groups'] = machines.get_all_groups()
        return c
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))

def get_server_context(server_id):
    """ Получение контента для определенного сервера
    """ 
    try:
        c = {}
        c['server_id'] = server_id
        c['server_name'] = machines.get_name(server_id)
        c['status'] =  machines.get_status(server_id)
        c['installations'] = installations.get_installations(server_id)
        c['installations_by_time'] = installations.get_by_time(server_id)
        c['server_groups'] = c['groups_to_remove'] = machines.get_groups(server_id)
        c['groups_to_add'] = machines.get_groups_not_in(server_id)
        c['server_attrs'] = machines.get_attrs(server_id)
        return c
    except Exception as e:
        logging.error(e)

def login_user_page(request, params=''):
    """ Пользователь отправляет запрос на вход.
    """ 
    try:
        c = {}
        c.update(csrf(request))
        if request.method == 'POST':
            username = request.POST.get('username', '')
            password = request.POST.get('password', '')
            user = auth.authenticate(username = username, password = password)
            if user is not None:
                if user.is_active:
                    login(request, user)
                    return HttpResponseRedirect("/")
            c['err_message'] = 'Неправильный логин и/или пароль'
            return render_to_response('auth.html', c)
        else:
            return render_to_response('auth.html', c)
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))
        return HttpResponseRedirect("/")
    
def logout_user_page(request):
    """ Пользователь отправляет запрос на выход.
    """ 
    try:
        logout(request)
        return HttpResponseRedirect("/auth")
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))
        return HttpResponseRedirect("/")
    
@login_required
def home_page(request):
    """ Запуск сервера, который связывается с агентами, если он не запущен. 
    """ 
    try:
        c = get_common_context(request)
        if 'filter_by_group' in request.GET:
            filter_ = request.GET['filter_by_group']
            if filter_ == u'Выделить все':
                for i in range(0, len(c['servers'])):
                    c['servers'][i]['checked'] = True
            elif filter_ == u'Снять все':
                pass
            else:
                machines_in_group = machines.get_machines_in_group(filter_)
                for i in range(0, len(c['servers'])):
                    if c['servers'][i]['id'] in machines_in_group:
                        c['servers'][i]['checked'] = True
            c['filter_by_group'] = filter_
        return render_to_response('home.html', c)
    except Exception as e:
        logging.error(cgitb.text(sys.exc_info()))

def has_permissions(user, server_id):
    return get_user_level(user) >=  machines.get_level(server_id)

""" *** Представления для страницы с серверами (/servers, /installation) *** """
@login_required
def servers_page(request, server_id):
    """ Страница с серверами
    """
    try:
        c = get_common_context(request)
        allowed_access = has_permissions(request.user, server_id)
        if not allowed_access:
            c['err_msg'] = "У Вас нет доступа к этому серверу"
        
        if request.method == 'GET':
            """ просто получение всей информации
            """
            c.update(get_server_context(server_id))
            return render_to_response('servers.html', c)
        else:
            """ POST запрос
            """
            action = request.POST.get('action', '')
            if action == '':
                logging.error('Empty request at servers page')
                return HttpResponseRedirect("/")
            elif action == u'Отмена':
                """ Нажата отмена, возвращаем страницу как при GET                
                """
                return HttpResponseRedirect("/servers/" + server_id)
            elif action == u'Удалить':
                """ Удаление сервера из базы данных 
                    и отправление ему запроса на отключение
                """
                if allowed_access:
                    machines.delete(server_id)
                    machines.send_exit_request(server_id)
                    return HttpResponseRedirect('/')
                else:
                    return render_to_response('servers.html', c)
            elif action == u'Переименновать':
                """ Запрос на переименнование сервера,
                    добавление на страницу поля ввода нового имени
                """
                if allowed_access:
                    c.update(get_server_context(server_id))
                    c['rename'] = 'Yes'
                return render_to_response('servers.html', c)
            elif action == u'Изменить уровень доступа':
                """ Запрос на изменение уровня доступа к серверу,
                    добавление на страницу поля ввода нового уровня
                """
                if allowed_access:
                    c.update(get_server_context(server_id))
                    c['change_level'] =  machines.get_level(server_id)
                return render_to_response('servers.html', c)
            elif action == u'Сохранить':
                """ Сохранение нового имени сервера
                """
                if allowed_access:
                    if request.POST['action_type'] == 'rename':
                        machines.rename(server_id, request.POST["new_name"])
                    elif request.POST['action_type'] == 'change_level':
                        machines.set_level(server_id, int(request.POST["new_level"]))
                    return HttpResponseRedirect("/servers/" + server_id)
                else:
                    return render_to_response('servers.html', c)
            elif action == u'Изменить' or action == u'Добавить':
                c.update(get_server_context(server_id))
                if request.POST['about'] == 'groups':
                    c['is_changing_groups'] = True     
                else: #request.POST['about'] == 'attrs'
                    c['is_changing_attrs'] = True 
                return render_to_response('servers.html', c)
            elif action == 'add_to_group':
                machines.set_group(server_id, request.POST['group_to_in'])
                c.update(get_server_context(server_id))         
                c['is_changing_groups'] = True        
                return render_to_response('servers.html', c)
            elif action == 'add_to_new_group':
                machines.set_group(server_id, request.POST['new_group'])
                c.update(get_server_context(server_id))              
                c['is_changing_groups'] = True   
                return render_to_response('servers.html', c)
            elif action == 'remove_from_group':
                machines.remove_from_group(server_id, request.POST['group_to_remove_from'])
                c.update(get_server_context(server_id))     
                c['is_changing_groups'] = True            
                return render_to_response('servers.html', c)
            elif action == 'add_attr':
                machines.set_attr(server_id, request.POST['name'], request.POST['default'])
                c.update(get_server_context(server_id))      
                c['is_changing_attrs'] = True         
                return render_to_response('servers.html', c)
            elif action == 'delete_attr':
                machines.delete_attr(server_id, request.POST['name'])
                c.update(get_server_context(server_id))          
                c['is_changing_attrs'] = True     
                return render_to_response('servers.html', c)
            else:
                """ Вероятнее всего здесь запрос на установку
                """
                if allowed_access:
                    c['is_installation'] = True
                    if action == u'Установить' or action == u'Переустановить':
                        return installation_begin(c, request.POST['recipe_id'], server_id)
                    elif action == u'Готово':
                        return installation_ready(c, request, server_id)
                    else:
                        if "server_name" in request.POST:
                            return HttpResponseRedirect("/servers/" + server_id)
                        else:
                            logging.error("Unknown request at servers page")
                            return HttpResponseRedirect('/')
                else:
                    return render_to_response('servers.html', c)
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))
        return HttpResponseRedirect("/")    
    
def installation_begin(c, recipe_id, server_id):
    """ Начало установки, запрос конфигов
    """
    try:        
        default_config = recipes.get_default_config(recipe_id, recipes.get_last_version(recipe_id))
        server_attrs = machines.get_attrs_as_dict(server_id)
        not_found_attrs = []
        for d in default_config:
            if d[2] == 'attr':
                if d[0] not in server_attrs:
                    not_found_attrs.append(d[0])
        if len(not_found_attrs) > 0:
            c['config_error_message'] = u"Для установки этого рецепта необходимо установить атрибуты сервера: " + ",".join(not_found_attrs)
        c['config'] = recipes.get_default_config(recipe_id, recipes.get_last_version(recipe_id), False)
        c['recipe_id'] = recipe_id
        c['is_installation'] = True
        c['getting_config'] = True
        c.update(get_server_context(server_id))
        return render_to_response('servers.html', c)
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))
        return HttpResponseRedirect("/")

def installation_ready(c, request, server_id):
    """ Отправка агенту запрос на установку
    """
    try:
        recipe_id = request.POST['recipe_id']
        recipe_version = recipes.get_last_version(recipe_id)
        c.update(get_server_context(server_id))
        
        config = {}
        for arg in request.POST:
            if '.' in arg:
                key = arg.split('.')[1]
                val = request.POST[arg]
                config[key] = val     
        default_config = recipes.get_default_config(recipe_id, recipe_version)
        server_attrs = machines.get_attrs_as_dict(server_id)
        hidden_config_keys = set()
        for c in default_config:
            if c[0] not in config:
                if c[2] == 'attr':
                    if c[0] in server_attrs:
                        config[c[0]] = server_attrs[c[0]]
                    else:
                        logging.error("Can't got necessary server attribute " + c[0])
                        return HttpResponseRedirect('/')
                else:
                    config[c[0]] = ''
            if c[2] == 'password':
                hidden_config_keys.add(c[0])
        
        if machines.send_installation_request(server_id, recipe_id, recipe_version, config, hidden_config_keys):
            return HttpResponseRedirect("/".join(["/installation", server_id, recipe_id, str(recipe_version)]))
        else:
            logging.error("Unable to send installation request to agent " + server_id)
            return HttpResponseRedirect('/')
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))
        return HttpResponseRedirect("/")
    
@login_required
def installation_page(request, server_id, recipe_id, recipe_version):
    """ Инсталляция в процессе.
        Вывод страницы с серверами с диалогом, в котором обновляется лог
    """
    try:
        c = get_common_context(request)
        c['log'] = installations.get_recent_installation_log(server_id)   
        c['is_installation'] = True
        c.update(get_server_context(server_id))
        c['status'] = 'Installation'
        return render_to_response('servers.html', c)
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))
        return HttpResponseRedirect("/")
    
@login_required
def multi_installing_page(request):
    """ Запрос на установку на несколько серверов.
    """
    try:
        c = get_common_context(request)
        servers = []
        if request.method == 'POST':
            action = request.POST.get('action', '')
            
            if action != u'Готово':
                for key in dict(request.POST):
                    if str(key).startswith('install_'):
                        recipe_id = key[8:]
                servers = request.POST.getlist('servers_to_install')
                c['config'] = recipes.get_default_config(recipe_id, recipes.get_last_version(recipe_id))
                c['recipe_id'] = recipe_id
                c['servers_to_install'] = servers
                c['is_installation'] = True
            elif action == u'Готово':
                recipe_id = request.POST['recipe_id']
                servers = eval(request.POST['servers_to_install'])
                recipe_version = recipes.get_last_version(recipe_id)
                config = {}
                default_config = recipes.get_default_config(recipe_id, recipe_version)
                hidden_config_keys = []
                for d_c in default_config:
                    if d_c[1] == '':
                        config[d_c[0]] = d_c[1]
                        if d_c[2] == 'password':
                            hidden_config_keys.append(d_c[0])
                for arg in request.POST:
                    if '.' in arg:
                        key = arg.split('.')[1]
                        val = request.POST[arg]
                        config[key] = val     
                
                for server_id in servers:
                    if machines.send_installation_request(server_id, recipe_id, recipe_version, config, hidden_config_keys):
                        return HttpResponseRedirect("/".join(["/installation", server_id, recipe_id, str(recipe_version)]))
                    else:
                        logging.error("Unable to send installation request to agent " + server_id)
                        return HttpResponseRedirect('/')
                return HttpResponseRedirect('/')
        return render_to_response('home.html', c)
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))
        logging.error("error in multi installing")
        return HttpResponseRedirect('/')
    

""" *** Представления для страницы с рецептами (/recipes) *** """
@login_required
def recipes_page(request):
    """ Страница с рецептами, просто меню с рецептами, без поля для редактирования
    """
    try:
        c = get_common_context(request)
        return render_to_response('recipes.html', c)
    except Exception as e:
        logging.error(e)


@login_required
def recipe_edit_page(request, recipe_id, recipe_version):
    """ Страница с рецептами, вместе с меню для редактирования
    """
    try:
        c = get_common_context(request)
        if request.method == 'GET':
            """ шаг 1. Вывод содержимого рецепта
            """
            if recipe_id == 'new':
                c['recipe_name'] = 'new'
                if recipes.exists('new'):
                    recipe_version = recipes.get_next_version(recipes.get_id('new'))
                c['recipe_version'] = recipe_version
                file_content = recipes.get_recipe_template()
                c['file_content'] = junta_xml.preprocessor(file_content, False)
            else:
                c['recipe_name'] = recipes.get_name(recipe_id)
                c['recipe_version'] = recipe_version
                file_content = recipes.get_recipe_content(recipe_id, recipe_version)
                c['file_content'] = junta_xml.preprocessor(file_content, False)
            return render_to_response('recipes.html', c)
        
        elif request.POST['action'] == u'Сохранить':
            """ шаг 2. Сохранение изменений
            """
            file_content = request.POST["content"]
            file_content = junta_xml.preprocessor(file_content)
            err_msg = recipes.not_valid_error_message(file_content)
            recipe_name = request.POST.get('recipe_name', 'unknown')
            
            if err_msg is None:
                """ если документ валиден
                """
                if not recipes.exists(recipe_name):
                    recipe_id = recipes.create(recipe_name)
                    recipes.create_recipe_path_if_not_exists(recipe_id)
                err_msg = recipes.save(recipe_name, file_content, request.user.username)
                recipe_version = recipes.get_last_version(recipe_id)
                if err_msg is None:
                    """ если не произошло ошибки при сохранении
                    """
                    return HttpResponseRedirect('/recipes/' + recipe_id + '/' + str(recipe_version))
            """ если произошла какая-то ошибка, возврат
            """
            c['err_msg'] = err_msg
            c['recipe_name'] = recipe_name
            c['recipe_version'] = recipe_version
            c['file_content'] = junta_xml.preprocessor(file_content, False)
            return render_to_response('recipes.html', c)
        
        elif request.POST['action'] == u'Назад':   
            return render_to_response('recipes.html', c)
        elif request.POST['action'] == u'Удалить':   
            if recipes.get_version_count(recipe_id) <= 1: # это был последний рецепт в своем роде
                recipes.recipe_delete(recipe_id)
                return HttpResponseRedirect("/recipes")
            else:
                recipes.delete_recipe_version(recipe_id, recipe_version)
                c = get_common_context(request)
                last_version = recipes.get_last_version(recipe_id)
                return HttpResponseRedirect("/".join(["/recipes", recipe_id, str(last_version)]))
        else:
            logging.error('Unknown request at recipes page: ' + str(request.POST['action']))
            return HttpResponseRedirect("/")
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))
        return HttpResponseRedirect("/")
    
@login_required
def add_recipe_page(request):
    """ Запрос на добавление рецепта, перенаправление
        имя и версия по умолчанию,содержимое из шаблона
    """
    return recipe_edit_page(request, 'new', 1)